package fourmi;

public abstract class Fourmi{
    private final static FourmiIdent usine = new FourmiIdent();

	//la classe FourmiIdent sert à fournir les identifiants uniques
	//un bête static int compteur à la place est acceptable
	//remarquez que à la différence du compteur int static, on peut ici mettre 
	//final en plus de static (réfléchissez pour savoir pourquoi!)


    private final int id;
	//final, c'est mieux!

    private int age;

    public Fourmi(int age){
	//const utile même pour classe abst
	//le paramètre "int" est un peu dangereux : 
	//	1) ce constructeur ne devrait être appelé qu'avec des ages de 0 ou 10,
	//	ce qui est garanti car on ne peut PAS appeler ce constsructeur directement 
	//	(on est obligé de l'appeler via des constructeurs des classes héritées), et 
	//	les constsructeurs des classes héritées, eux, n'ont pas de paramètre int age
	//	2) cependant, un type énum à la place de int aurait pu être possible

	this.id = this.usine.next();
	this.age = age;
    }

    Fourmi(int futurId, int age){
	//ce constructeur est un peu dangeureux aussi, et n'est donc pas public (mais limité au package)
	//il sert uniquement lors du transform : une larve devient adulte en conservant son ID
	this.id = futurId;
	this.age = age;
    }




    public abstract Role role();

    int getId(){
	//rappel : pas de modificateur = portée limitée au package
	return this.id; 	
    }


    public String toString(){
	// methode non abstract utile même dans classe abst

	return ("id = " + id + " age = " + age + " / " + this.getAgeMaximum() + " Role = " + 
		this.role().toString());
	
	//1) la liaison dynamique assurera que le "this.role()" et le "this.getAgeMaximum()" choisira la "bonne méthode"
	//2) role() retourne un objet de type Role, qui est une énum, 
	// et les enum héritent d'un toString() par défaut qui affiche le nom de l'étiquette
    }

    ///////////////////////////////////////
    /////    partie gestion de l'age  /////
    ///////////////////////////////////////

    public int getAge(){
       return this.age;
    }

    abstract int getAgeMaximum();

    public boolean testAgeMaxi(){
		return (this.getAge() >= this.getAgeMaximum());	
    }

    public void viellir() throws ViellirException{
	//j'ai changé le type de retour en "void"
	//invariant : qd on appelle viellir, this.age < this.getAgeMaximum();

	if (!(this.getAge() < this.getAgeMaximum())){
		throw new ViellirException(this);        
	}
    	this.age++;
    }

    ///////////////////////////////////////
    /////    partie gestion de l'évol./////
    ///////////////////////////////////////


    public abstract FourmiAdulte transform();
	//chez les larves : transform transforme en adulte
	//chez les adultes : transform retourne null <=> mort

    public Fourmi evoluer() throws ViellirException{
    	//invariant : avant appel this.age < this.getAgeMaximum();
	if (!(this.getAge() < this.getAgeMaximum())){
		throw new ViellirException(this);        
	}
	this.viellir();
	if (this.testAgeMaxi()){
		return this.transform();	
	}
	else{
		return this;	
	}	
    }

}
	
